
// main.cpp
////////////

// QT Includes und Standard Defines
#include "qincludes.h"
// Weitere Definitionen von MyLineEdit einbinden
#include "mylineedit.h"
#include "mystack.h"
#include "mycalc.h"
#include "myhelp.h"
#include "myexception.h"



int main( int argc, char** argv )
{
	// Das Hauptprogramm
	QApplication *app = new QApplication( argc, argv );

	// Hauptfenster
	QMainWindow *window = new QMainWindow( 0, PROGRAM_SHORT );
	// Zentrales Widget an welches das gesamte Layout geknuepft ist.
	QFrame *central = new QFrame( window );
	// Sublayout der Eingabezeile
	QBoxLayout *entryline = new QHBoxLayout ( -1, "SubLayout" );
	// Hilfeknopf
	QPushButton *helpbutton = new QPushButton( central );
	// Eingabezeile
	MyLineEdit *newentry = new MyLineEdit( central );
	// Stapelspeicher
	MyStack *stack = new MyStack( central );
	// Informationszeile
	QLabel *message = new QLabel( central );
	// Rechnerabstraktion
	MyCalc *calc = new MyCalc( stack, message );
	// Hilfeseite
	MyHelp *helpwindow = new MyHelp( central, PROGRAM_SHORT ": Hilfe" );

	// Eigenschaften fuer Hilfebutton setzen
	// Inhalt
	helpbutton->setText( QString("?") );
	// Groesse
	helpbutton->setMaximumSize( 18, 24 );
	helpbutton->setMinimumSize( 18, 24 );

	// Sublayout fuer Eingabezeile
	// a. Eingabefeld
	entryline->addWidget( newentry );
	// b. Hilfebutton
	entryline->addWidget( helpbutton );

	// Gesamt Layout zusammenbauen
	stack->setLayout( central, entryline, message );

	// Schriftart fuer alles setzen (Courier ist anti-proportional)
	window->setFont( QFont("Courier",12 ));

	// Eingabezeile mit dem Rechenwerk verbinden
	window->connect( newentry, SIGNAL(returnPressedValue(QString)), calc, SLOT(apply(QString)) );

	// ? mit der Hilfe verbinden.
	window->connect( helpbutton, SIGNAL(clicked()), helpwindow, SLOT(toggle()) );

	// Klick auf Stack kopiert Daten in und setzt den Fokus wieder auf die Eingabezeile
	window->connect( stack, SIGNAL(selectionChanged()), newentry, SLOT(setFocus()) );
	window->connect( stack, SIGNAL(clickedText(QString)), newentry, SLOT(setLine(QString)) );

	// Fokus auf Eingabezeile setzen
	newentry->setFocus();

	// alle Tastatureingaben gehen nur auf die Eingabezeile
	newentry->grabKeyboard();
	
	// Willkommensnachricht setzen
	calc->setStatus("");

	// zentrales Widget festlegen
	window->setCentralWidget( central );

	// Applikationsfenster festlegen
    app->setMainWidget( window );

	// Fenster anzeigen
	window->show();
                                                                   
	// Applikation ausfuehren
	return app->exec();
};
