
// mycalc.cpp
//////////////

// Header Dateien einbinden
#include "mycalc.h"
#include "myexception.h"
#include <math.h>
#include <stdlib.h>
#include <errno.h>


// Konstante fuer PI
const double MyCalc::pi = PI_VALUE;


MyCalc::MyCalc( MyStack *stack_, QLabel *message_ )
{
	/**
	 * Konstuktor
	 */

	// Pointer kopieren
	stack     = stack_;
	message   = message_;

	// Variablen initialisieren
	errormsg = false;

	// Standardgenauigkeit festlegen
	stack->setPrecision( DEFAULT_PRECISION );
};


void MyCalc::apply( QString command )
{
	/**
	 * Das eigentliche Rechenwerk: den Eingabeparameter anwenden
	 */

	// Rechenwerte
	double x = 0, y = 0;
	// Ergebnis
	double result = 0;
	// Mehrfach verwendeter Status
	bool ok = false;
	// Status fuer hoehere Rechenfunktionen zuruecksetzen
	errno = 0;

	// command in Kleinschrift umwandeln, fuer mehr Benutzerfreundlichkeit
	command = command.lower();

	// das eigentliche Rechenwerk
	try
	{
		// Soll das Programm beendet werden?
		if(command == "quit")
		{
			// offensichtlich, oder?
			exit(0);
		}

		// End of Line - Kennzeichen
		if(command == QChar(1))
		{
			// Trat ein Fehler auf?
			// das Folgende zweistufen-Prinzip verhindert, dass Fehlermeldungen durch die
			// Defaultzeile ueberschrieben werden
			if( errormsg )
			{
				// ja, Fehlerkennung loeschen
				errormsg = false;
			}
			else
			{
				// nein, Statusanzeige loeschen
				setStatus( "" );
			}
			// weitere Auswertung des Befehls verhindern
			return;
		}

		// Rechenrichtung umkehren?
		if((command == "mode") || (command == "view"))
		{
			// Umkehrung ausloesen
			stack->toggleMode();
			// weitere Auswertung des Befehls verhindern
			return;
		}

		// Element vom Stapel entfernen?
		if((command == "drop") || (command == "\\"))
		{
			// Element vom Stapel holen
			stack->pull();
			// weitere Auswertung des Befehls verhindern
			return;
		}
	
		// letzten Wert duplizieren?
		if((command == "dup") || (command == "="))
		{
			// Wert vom Stapel holen
			QString dummy = stack->pull();
			// zweimal wieder draufschreiben
			stack->push(dummy);
			stack->push(dummy);
			// weitere Auswertung des Befehls verhindern
			return;
		}
	
		// letzten beiden Werte vertauschen
		if((command == "swap") || (command == "#"))
		{
			// Werte holen
			QString dummy0 = stack->pull();
			QString dummy1 = stack->pull();
			// in umgekehrter Reihenfolge wieder drauf
			stack->push(dummy0);
			stack->push(dummy1);
			// weitere Auswertung des Befehls verhindern
			return;
		}
	
		// in hex konvertieren?
		if((command == "hex") || (command == "$"))
		{
			// Wert vom Stapel holen
			QString dummy=stack->pull();
			// und wieder zurueckschreiben
			stack->push(dummy);
			// konvertieren
			dummy="0x" + QString::number( dummy.toULong(&ok), 16 );
			// Konvertierung ok?
			if(!ok)
			{
				// nein, Fehlermeldung
				throw MyException( "nicht darstellbar" );
			}
			// ja, Wert als Hexzahl in Statuszeile
			setStatus(dummy);
			// weitere Auswertung des Befehls verhindern
			return;
		}
	
		// Rechengenauigkeit festlegen?
		if((command == "prec") || (command == "scale"))
		{
			// Wert vom Stack als Ganzzahl holen (Sonderfall)
			int newprecision = stack->pull().toInt(&ok);

			// war Konvertierung korrekt?
			if(ok)
			{
				// liegt der Wert zwischen 1 und 16?
				if((newprecision < 1) || (newprecision > 16))
				{
					ok = false;
				}
			}
			// Wert ok?
			if(!ok)
			{
				// Wert nicht ok
				throw MyException( "Rechengenauigkeit falsch" );
			}
			// Genauigkeit setzen
			stack->setPrecision( newprecision );
			// Statusmeldung ausgeben
			setStatus( "Rechengenauigkeit: " + 
						QString::number( newprecision ) + " Stellen" );
			// weitere Auswertung des Befehls verhindern
			return;
		}
	
		// addieren?
		if((command == "add") || (command == "+"))
		{
			// Werte vom Stapel holen
			x = stack->pullDouble();
			y = stack->pullDouble();
			// addieren
			result = y + x;
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// subtrahieren?
		if((command == "sub") || (command == "-"))
		{
			// Werte vom Stapel holen
			x = stack->pullDouble();
			y = stack->pullDouble();
			// subtrahieren
			result = y - x;
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// multiplizieren?
		if((command == "mul") || (command == "*"))
		{
			// Werte vom Stapel holen
			x = stack->pullDouble();
			y = stack->pullDouble();
			// multiplizieren
			result = y * x;
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// dividieren?
		if((command == "div") || (command == "/"))
		{
			// Werte vom Stapel holen
			x = stack->pullDouble();
			y = stack->pullDouble();
			// Bei Division durch null Fehler werfen
			if( x == 0 ) throw MyException( "Division durch 0" );
			// dividieren
			result = y / x;
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Divisionsrest bestimmen?
		if((command == "mod") || (command == "%"))
		{
			// Werte vom Stapel holen
			x = stack->pullDouble();
			y = stack->pullDouble();
			// Bei Division durch null Fehler werfen
			if( x == 0 ) throw MyException( "Division durch 0" );
			// Divisionsrest bestimmen
			result = fmod(y, x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// potenzieren?
		if((command == "pow") || (command == "^"))
		{
			// Werte vom Stapel holen
			x = stack->pullDouble();
			y = stack->pullDouble();
			// potenzieren
			result = pow(y, x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Quadratwurzel ziehen
		if((command == "sqrt"))
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			result = sqrt(x);
			// sqrt() will keine Fehlermeldung setzen, also von Hand!
			if( x < 0 ) errno = EDOM;
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Wurzel ziehen
		if((command == "xrt"))
		{
			// Werte vom Stapel holen
			x = stack->pullDouble();
			y = stack->pullDouble();
			// geschicktes potenzieren liefert die passende Wurzel
			result = pow(y, 1/x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Sinus?
		if(command == "sin")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Sinus
			result = sin(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Cosinus?
		if(command == "cos")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Cosinus
			result = cos(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Tangenz?
		if(command == "tan")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Tangenz
			result = tan(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Arcussinus?
		if(command == "asin")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Arcussinus
			result = asin(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Arcuscosinus?
		if(command == "acos")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Arcuscosinus
			result = acos(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Arcustangenz?
		if(command == "atan")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Arcustangenz
			result = atan(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Sinus Hyperbolicus?
		if(command == "sinh")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Sinus Hyperbolicus
			result = sinh(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Cosinus Hyperbolicus?
		if(command == "cosh")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Cosinus Hyperbolicus
			result = cosh(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Tangenz Hyperbolicus?
		if(command == "tanh")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Tangenz Hyperbolicus
			result = tanh(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// e^x?
		if(command == "exp")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// e^x
			result = exp(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Logarythmus zur Basis e?
		if(command == "ln")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Logarythmus zur Basis e
			result = log(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Logarythmus zur Basis 10?
		if(command == "log")
		{
			// Wert vom Stapel holen
			x = stack->pullDouble();
			// Logarythmus zur Basis 10
			result = log10(x);
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		if(command == "pi")
		{
			// "Rechenergebnis" als Wert von PI setzen
			result = pi;
			// Status zeigt an, dass result gueltig ist
			ok = true;
		}

		// Ueberpruefen, ob er beim Rechnen Fehler aufgetreten sind
		if(ok)
		{
			// Ergebnis undefiniert?
			if( errno == EDOM )
			{
				// passende Fehler werfen
				throw MyException( "Ergebnis undefiniert" );
			}
			// Ergebnis ausserhalb des Wertebereiches?
			else if( errno == ERANGE )
			{
				// passende Fehler werfen
				throw MyException( "Ergebnis nicht darstellbar" );
			}
			else
			{
				// Ergebnis auf dem Stapel ablegen
				stack->push( result );
			}
		}
		else
		{
			// ist der Befehl eine Zahl?
			command.toDouble(&ok);
			if(ok)
			{
				// ja, dann die Zahl auf den Stapel legen
				stack->push( command );
			}
			else
			{
				// nein, Fehler werfen
				throw MyException( "Eingabefehler" );
			}
		}
	}
	catch (MyException exception)
	{
		// Fehler fangen, und als Meldung auf der Statuszeile ausgeben
		setStatus( exception.getMessage() );
	}
};


void MyCalc::setStatus( QString text )
{
	/**
	 * Statuszeile setzten
	 */

	// ist der Text leer?
	if(text == "")
	{
		// ja, dann Defaulttext (Programmname) setzen
		message->setText(QString("=> " PROGRAM_SHORT ": " PROGRAM_LONG " <="));
	}
	else
	{
		// nein, Fehlertext setzen
		message->setText(text);
		// Statuszeile enthaelt eine Fehlermeldung
		errormsg = true;
	}
	// Zeile zentrieren
	message->setAlignment( Qt::AlignCenter );
};
