
// mylineedit.cpp
//////////////////

#include "mylineedit.h"


MyLineEdit::MyLineEdit(QWidget *parent, const char *name ) : QLineEdit (parent, name)
{
	/**
	 * Konstruktor, mit QLineEdit als Basisklasse
	 * Ableitung wird nur benoetigt, um Signals und Slots zu verwenden
	 */

	// Eingabe rechtsbuendig
	this->setAlignment( Qt::AlignRight );
	// Return-Taste abfangen
	this->connect(this, SIGNAL(returnPressed()), this, SLOT(returnPressedCatch()));
};


void MyLineEdit::tryEmit(QString emitMe)
{
	/**
	 * Nur etwas weiter schicken, wenn auch wirklich Daten vorhanden sind
	 */

	// ist der String leer?
	if(!emitMe.isEmpty())
	{		
		// nein, dann wegschicken
		emit returnPressedValue(emitMe);
	}
};


void MyLineEdit::returnPressedCatch()
{
	/**
	 * Diese Methode fngt die Eingabetaste ab, und sendet den Inhalt weiter.
	 * Nur nicht leere Zeichenketten werden weiter gesandt
	 * Ausserdem wird an Leerzeichen umgebrochen...
	 */

	// Eingabezeile fuers Bearbeiten kopieren
	QString myText   = this->text();

	// Hilfsvariablen zum Analysieren des Strings
	unsigned int first = 0, last = 0, end = myText.length();

	// Auf dem Strings langlaufen
	for(last=first=0; last < end; last++)
	{
		// ist das aktuelle Zeichen ein Komma?
		if(myText.at(last) == ',')
		{
			// wenn ja, dann durch einen Punkt ersetzen
			myText = myText.remove( last, 1 );    // Komma entfernen
			myText = myText.insert( last, '.' );  // Punkt dafuer einfuegen
		}
		// ist das aktuelle Zeichen ein Leerzeichen?
		if(myText.at(last) == ' ')
		{
			// String bis hierher wegschicken
			tryEmit(myText.mid(first,last-first));
			// neuen Anfang suchen
			for(first=last; myText.at(first) == ' '; first++);
			// aktuelles Zeichen als erstes nach dem letzten gefundenen Leerzeichen setzen
			last=first;
		}
	}
	// Rest verschicken
	tryEmit(myText.mid(first,last-first));

	// End of Line Kennung verschicken
	tryEmit( QChar(1) );

	// Eingabezeile wieder loeschen
	this->validateAndSet(QString(""),0,0,0);
};


void MyLineEdit::setLine( QString text )
{
	/**
	 * Inhalt der Eingabezeile setzen
	 * (Ziel, wenn im Stapel auf ein Element geklickt wird)
	 */

	// Text setzen
	this->setText( text );
};
