
// mystack.cpp
///////////////

// Definition einbinden
#include "mystack.h"
#include "myexception.h"

void MyStackText::paint( QPainter *painter )
{   
	/**
	 * Abgeschrieben aus QListBoxText::paint, dann auf rechtsbuendig getrimmt
	 */

	// Hoehe der Zelle ermitteln
    int itemHeight = this->height( this->listBox() );
	// Fontbeschaffenheit uebernehmen
    QFontMetrics fm = painter->fontMetrics();
	// Y Position ausrechnen
    int yPos = ( ( itemHeight - fm.height() ) / 2 ) + fm.ascent();
	// X Position ausrechnen (nicht in original paint()-Methode vorhanden)
    int xPos = this->listBox()->contentsWidth() - this->width( this->listBox() );
	// Text rechtsbuendig zeichnen
    painter->drawText( xPos + 2, yPos, this->text() );
};



MyStack::MyStack(QWidget *parent, const char *name ) : QListBox (parent, name) 
{
	/**
	 * Konstruktor
	 */

	// Eingabezeile steht zum Beginn unten
	endAtTop = false;
	// Vector fuer Gesamtlayout auf leer setzen
	layout = NULL;
	// Vertikale Scrollbar ein
	this->setVScrollBarMode( QScrollView::AlwaysOn );
	// Horizontale Scrollbar aus
	this->setHScrollBarMode( QScrollView::AlwaysOff );
	// Verbindung zum Abfangen eines Klicks in den Stack legen
	this->connect(this, SIGNAL(selectionChanged()), this, SLOT(clickedCatch()));
};

MyStack::~MyStack()
{
	/**
	 * Destruktor
	 */

	// Wenn ein layout existiert, dieses Loeschen
	if(layout) delete layout;
};

void MyStack::setPrecision( int precision_ )
{
	/**
	 * Genauigkeit festlegen, mit dem double-Werte auf den Stack gelegt werden
	 */

	// Wert umkopieren
	precision = precision_;
};

void MyStack::setLayout( QWidget *central_, QBoxLayout *entryline_, QLabel *message_ )
{
	/**
	 * Layout erzeugen.
	 * Da nur die Stackklasse "weiss", in welche Richtung das Rechenwerk arbeitet,
	 * muss sie auch das Layout festlegen.
	 */

	// Beim Aufruf aus dem Hauptprogramm Vectoren merken
	if(central_)	central	  = central_;
	if(entryline_)	entryline = entryline_;
	if(message_)	message	  = message_;

	// Gesamtlayout erzeugen
	// altes vorhanden?
	if(layout)
	{
		// wenn ja, dann loeschen
		layout->removeChild( entryline );
		delete layout;
	}
	// neues erzeugen
	layout = new QVBoxLayout( central, 0, -1, "MainLayout" );

	// je nach Ausrichtung des Rechenwerks das Layout setzen
	if(endAtTop)
	{
		// 1. Eingabezeile
		layout->addLayout( entryline );
		// 2. Stack
		layout->addWidget( this );
	}
	else
	{
		// 1. Stack
		layout->addWidget( this );
		// 2. Eingabezeile einfuegen
		layout->addLayout( entryline );
	}
	// 3. Statuszeile ist immer unten
	layout->addWidget( message );

	// neues Layout aktivieren (Sonntag morgen um 3 Uhr endlich gefunden ;-)
	layout->activate();
}

void MyStack::goToEnd()
{
	/**
	 * Unterroutine, die dafuer sorgt, dass an das Ende des Stacks gesprungen
	 * wird welches - je nach Rechenwerkeinstellung - oben oder unten ist.
	 */

	// Rechnet das Rechenwerk von oben?
	if(endAtTop)
	{
		// ja, aktuelles Element (Ende) ist oben
		this->setCurrentItem( 0 );
	}
	else
	{
		// nein, aktuelles Element (Ende) ist unten
		this->setCurrentItem( this->count()-1 );
	}
	// darauf zentrieren
	this->centerCurrentItem();
	// selection wieder loeschen
	this->clearSelection();
};

void MyStack::push( QString pushMe )
{
	/**
	 * Eine Zeile auf den Stapel legen
	 */

	// Rechnet das Rechenwerk von oben?
	if(endAtTop)
	{
		// oben Einfuegen
		this->insertItem( new MyStackText( pushMe ), 0 );
	}
	else
	{
		// unten Einfuegen
		this->insertItem( new MyStackText( pushMe ), -1 );
	}
	// das Ende anzeigen
	goToEnd();
};

void MyStack::push( double pushMe )
{
	/**
	 * Einen double-Wert auf den Stapel legen
	 */

	// einfach nur konvertieren und den String dann auf den Stapel legen
	push( QString::number( pushMe, 'g', precision ) );
};

QString MyStack::pull()
{
	/**
	 * Eine Zeile vom Stapel holen
	 * wenn der Stapel leer ist, wird eine Exception geworfen
	 */

	// Rueckgabewert deklarieren
	QString retval;

	// Hast der Stapel Inhalt?
	if(this->count())
	{
		// Rechnet das Rechenwerk von oben?
		if(endAtTop)
		{
			// ja, oberstes Element zurueckgeben...
			retval = this->text( 0 );
			// ... und loeschen
			this->removeItem( 0 );
		}
		else
		{
			// nein, oberstes Element zurueckgeben...
			retval = this->text( this->count()-1 );
			// ... und loeschen
			this->removeItem( this->count()-1 );
		}
	}
	else
	{
		throw MyException( "Leerer Stack" );
	}
	// das Ende anzeigen
	goToEnd();
	return retval; 
};

double MyStack::pullDouble()
{
	/**
	 * double-Wert vom Stack holen
	 */

	// String vom Stapel holen und diesen dann konvertiert zurueckgeben
	return this->pull().toDouble();
};

bool MyStack::isEmpty()
{
	/**
	 * Ist der Stack leer?
	 */

	return this->count() == 0;
};

void MyStack::toggleMode()
{
	/**
	 * Position der Eingabezeile und den Rechenmodus aendern
	 */

	// Rechenwerk umkehren
	endAtTop = !endAtTop;
	// zum Ende springen
	goToEnd();
	// veraendertes Layout setzen
	setLayout();
};

void MyStack::clickedCatch()
{
	/**
	 * Text bei Klick in Stack ans Eingabefeld schicken
	 */

	// Selektion (blauer Balken) loeschen
	this->clearSelection();
	// Fokus (gestrichelter Rahmen) loeschen
	this->clearFocus();
	// wenn an der Stelle Text ist...
	if( !this->currentText().isEmpty() )
	{
		// ... diesen an der Eingabefenster schicken
		emit clickedText( this->currentText() );
	}
};
