
// mystack.h
/////////////

#ifndef _MYSTACK_H_
#define _MYSTACK_H_

#include "qincludes.h"

class MyStackText : public QListBoxText
{
	/**
	 * Ein Eintrag auf dem Stack, der rechtsbuendig dargestellt wird.
	 * Ableitung der QListBoxText-Klasse notwendig,
	 * um die paint()-Methode zu ueberladen.
	 */
public:
	// Konstruktor und Destruktor werden einfach durchgereicht.
	MyStackText( const QString &text ):QListBoxText( text ) {};
	~MyStackText() { /* destructor empty */ };
	// neue paint()-Methode (rechtsbuendig) (einziger Grund fuer diese Klasse)
	void paint( QPainter *painter );
};

class MyStack : public QListBox
{
	/**
	 * ListBox um die Funktionen erweitert damit ein Stapel angezeigt wird.
	 * Die Elemente auf dem Stapel sind Zeichenketten, keine Zahlen(!).
	 */
	Q_OBJECT
private:
	// Genauigkeit der gespeicherten double-Werte
	int precision;
	// Richtung des Rechenwerkes
	bool endAtTop;
	// Gesamtlayout
	QBoxLayout *layout;
	// gespeicherter Wert fuer zentrales Widget
	QWidget *central;
	// gespeicherter Wert fuer Eingabezeile
	QBoxLayout *entryline;
	// gespeicherter Wert fuer Nachrichtenzeile
	QLabel *message;
	void goToEnd();
public:
	MyStack(QWidget *parent, const char *name=0 );
	~MyStack();
	// Ein Element auf den Stapel legen.
	void push( QString pushMe );
	void push( double pushMe );
	// Ein Element vom Stapel holen
	QString pull();
	double pullDouble();
	// Abfrage, ob der Stapel leer ist
	bool isEmpty();
	// Genauigkeit der gespeicherten double-Werte festlegen
	void setPrecision( int precision_ );
	// Aendert die "Richtung" des Rechenwerkes
	void toggleMode();
	// Hauptlayout setzen
	void setLayout( QWidget *central_=0, QBoxLayout *entryline_=0, QLabel *message_=0 );
public slots:
	// Klick in Stapel abfangen (zum Kopieren in Eingabezeile)
	void clickedCatch();
signals:
	// Text senden, auf den geklickt wurde
	void clickedText( QString );
};

#endif
